#!/bin/bash

HELP_INFO='''
# Before Usage:
# FIPS_MODE environment variable needs to be set to one of the following disabled/enabled/strict
# More info for each mode can be found here - https://confluence.eng.vmware.com/display/CAEng/FIPS+Implementation+Notes
#
# Usage:
# source /usr/local/bin/setup_security.sh
# or
# source /usr/local/bin/setup_security.sh --external-cert-path=<SOME_PATH>
#
# Default --enternal-cert-path - /run/external-certs
#
# After that the following environment variables will be available for use
# * TRUST_STORE_PASSWORD - *deprecated* - contains the generated truststore password, will not be
#   changed if passed from outside. Will be removed in later versions.
# * JAVA_OPTS - will append the options needed to enable BC FIPS.
# * BCFIPS_JARS - the full path to bc-fips.jar or jars separated with :, so this can be added to
#   the start of application classpath like
#        java -cp ${BCFIPS_JARS:+"$BCFIPS_JARS":}$org_class_path
#
# The following files are located in the directory pointed by LIB_SECURITY_PATH
# * cacerts.pass - contains the password for the truststore in a format accepted by the Java keytool
'''

function print_help() {
  cat <<EOF
$HELP_INFO
EOF
}

JAVA_FIPS_SECURITY_OPTS=""

JAVA_SECURITY_PATH="$JAVA_HOME/conf/security"

DEFAULT_JAVA_SECURITY_FILE="$JAVA_SECURITY_PATH/java.security"
OVERRIDE_JAVA_SECURITY_FILE="$JAVA_SECURITY_PATH/vmware-override-java.security"

FIPS_MODE_DISABLED="disabled"
FIPS_MODE_ENABLED="enabled"
FIPS_MODE_STRICT="strict"

FIPS_SECURITY_PROVIDERS='''security.provider.1=org.bouncycastle.jcajce.provider.BouncyCastleFipsProvider C:DEFRND[HMACSHA512(256)];HYBRID;ENABLE{ALL};
security.provider.2=org.bouncycastle.jsse.provider.BouncyCastleJsseProvider fips:BCFIPS
security.provider.3=com.sun.net.ssl.internal.ssl.Provider BCFIPS
security.provider.4=sun.security.provider.Sun'''


for i in "$@"
do
  case $i in
  -c=* | --external-cert-path=*)
    EXTERNAL_CERT_PATH="${i#*=}"
    shift
    ;;
  -h | --help)
    PRINT_HELP_INFO=YES
    shift
    ;;
  *)
    # unknown option
    echo "ERROR: Invalid argument '$i'!"
    print_help
    exit 127
    ;;
  esac
done

if [ -n "$PRINT_HELP_INFO" ]
then
  print_help
  exit 0
fi

# Defaults
EXTERNAL_CERT_PATH=${EXTERNAL_CERT_PATH:-/run/external-certs}
echo "EXTERNAL_CERT_PATH=${EXTERNAL_CERT_PATH}"

#-------------------------------  OPTIONS --------------------------------------
if [ ! -n "$FIPS_MODE" ]
then
  export FIPS_MODE="$FIPS_MODE_DISABLED"
  echo "WARNING: FIPS_MODE environment variable is not set! The default value is set to $FIPS_MODE"
else
  export FIPS_MODE=${FIPS_MODE,,}
fi
echo "FIPS_MODE=$FIPS_MODE"

if ! [[ "$FIPS_MODE" =~ ^($FIPS_MODE_DISABLED|$FIPS_MODE_ENABLED|$FIPS_MODE_STRICT)$ ]]
then
  echo "ERROR: FIPS_MODE environment variable ($FIPS_MODE) is not set correctly! Use $FIPS_MODE_DISABLED/$FIPS_MODE_ENABLED/$FIPS_MODE_STRICT"
  exit 2
fi

#--------------------------------- CHANGE PASSWORD -----------------------------
LIB_SECURITY_PATH="$JAVA_HOME/jre/lib/security"
test -e "$LIB_SECURITY_PATH/cacerts" || LIB_SECURITY_PATH="$JAVA_HOME/lib/security"
test -e "$LIB_SECURITY_PATH/cacerts" || { echo "Cannot find cacerts file in $JAVA_HOME" ; exit 1 ; }
echo "Found cacerts in $LIB_SECURITY_PATH"
ls -al "$LIB_SECURITY_PATH"

CACERTS_PATH="$LIB_SECURITY_PATH/cacerts"
STOREPASS_FILE="$LIB_SECURITY_PATH/cacerts.pass"

if [ -f "$STOREPASS_FILE" ]
then
  echo "cacerts.pass was already initialized"
else
  # create files as read/write only to owner
  (
    umask 077
    openssl rand -base64 40 > "$STOREPASS_FILE"
  )

  keytool -storepasswd -keystore "$CACERTS_PATH" -storepass changeit -new:file "$STOREPASS_FILE"
fi

read TRUST_STORE_PASSWORD < "$STOREPASS_FILE"
export TRUST_STORE_PASSWORD

#-------------------------------IMPORT CERTIFICATES-----------------------------
echo "Using external cert path - $EXTERNAL_CERT_PATH"

for cert in "$EXTERNAL_CERT_PATH"/*.pem
do
  if test -f "$cert"
  then
    echo "Importing $cert"
    keytool -import -trustcacerts -keystore "$CACERTS_PATH" \
        -storepass:file "$STOREPASS_FILE" \
        -noprompt \
        -alias "prelude-int-$(basename $cert)" \
        -file "$cert"
  fi
done

#-------------------------------SETUP FIPS--------------------------------------
# Define JAVA_OPTS if it is not yet defined
JAVA_OPTS=${JAVA_OPTS:-}

if [[ "$FIPS_MODE" =~ ^($FIPS_MODE_ENABLED|$FIPS_MODE_STRICT)$ ]]
then

  echo "Copying bouncycastle FIPS jar to $JAVA_HOME/lib..."
  cp $BCFIPS_DIR/bc*-fips-*.jar $JAVA_HOME/lib

  echo "Setup $OVERRIDE_JAVA_SECURITY_FILE with bouncycastle security providers..."
  # Create vmware-override-java.security file based on default java.security
  cp $DEFAULT_JAVA_SECURITY_FILE $OVERRIDE_JAVA_SECURITY_FILE
  # Remove all lines containing security.provider.<any number>
  sed -i.bak '/security.provider.[0-9]/d' $OVERRIDE_JAVA_SECURITY_FILE
  # Add FIPS list of providers with their preference order
  awk -v security_providers="$FIPS_SECURITY_PROVIDERS" '{print} /# List of providers and their preference orders.*/{print security_providers}' $OVERRIDE_JAVA_SECURITY_FILE > tmp_file

  # set default keystore type to bcfks
  echo "Set keystore.type to BCFKS"
  grep -q '^\s*keystore\.type\s*=' tmp_file || echo 'keystore.type=bcfks' >> tmp_file
  sed -i 's/\s*keystore\.type\s*=.*$/keystore.type=bcfks/' tmp_file

  echo "Set securerandom.strongAlgorithms to Java default one"
  grep -q '^\s*securerandom\.strongAlgorithms\s*=' tmp_file || echo 'securerandom.strongAlgorithms=NativePRNGBlocking:SUN' >> tmp_file
  sed -i 's/^\(securerandom\.strongAlgorithms\s*=\s*\).*$/\1NativePRNGBlocking:SUN/' tmp_file

  cat tmp_file > $OVERRIDE_JAVA_SECURITY_FILE && rm -f tmp_file

  if [ "$FIPS_MODE" == "$FIPS_MODE_STRICT" ]
  then
    JAVA_FIPS_SECURITY_OPTS+=" -Dorg.bouncycastle.fips.approved_only=true"
  fi
  JAVA_FIPS_SECURITY_OPTS+=" -Djava.security.properties==${OVERRIDE_JAVA_SECURITY_FILE}"
  BCFIPS_JARS=$( echo "$JAVA_HOME/lib/"bc*-fips-*.jar | tr ' ' ':' )
  echo "Defined BCFIPS_JARS='$BCFIPS_JARS' to be added java class path."

#-------------------------------CREATE BC STORE-----------------------------------------

  BCFKS_TRUSTSTORE="$LIB_SECURITY_PATH/truststore.bcfks"
  BCFKS_TRUSTSTOREPASS_FILE="$STOREPASS_FILE"

  keytool -importkeystore -srckeystore $CACERTS_PATH -destkeystore $BCFKS_TRUSTSTORE \
    -noprompt \
    -srcstoretype JKS -deststoretype BCFKS \
    -srcstorepass:file $STOREPASS_FILE -deststorepass:file $BCFKS_TRUSTSTOREPASS_FILE \
    -provider org.bouncycastle.jcajce.provider.BouncyCastleFipsProvider \
    -providerpath "$BCFIPS_JARS"

  JAVA_FIPS_SECURITY_OPTS+=" -Djavax.net.ssl.keyStoreType=BCFKS"
  JAVA_FIPS_SECURITY_OPTS+=" -Djavax.net.ssl.trustStoreType=BCFKS"
  JAVA_FIPS_SECURITY_OPTS+=" -Djavax.net.ssl.trustStore=$BCFKS_TRUSTSTORE"

  if [ -z "${JAVA_TLS_CLIENT_PROTOCOL}" ]
  then
    JAVA_TLS_CLIENT_PROTOCOL="TLSv1.2"
  fi
  JAVA_OPTS+=" -Djdk.tls.client.protocols=$JAVA_TLS_CLIENT_PROTOCOL"

  JAVA_OPTS+=" $JAVA_FIPS_SECURITY_OPTS"
  echo "'$JAVA_FIPS_SECURITY_OPTS' added to JAVA_OPTS environment..."

fi
#---------------------------------------------------------------------------------------
